@@ -1,13 +1,12 @@
 /* tc-d10v.h -- Header file for tc-d10v.c.
-   Copyright 1996, 1997, 1998, 2000, 2001, 2002, 2003, 2005
-   Free Software Foundation, Inc.
+   Copyright (C) 1996-2016 Free Software Foundation, Inc.
    Written by Martin Hunt, Cygnus Support.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -22,11 +21,7 @@
 
 #define TC_D10V
 
-#define TARGET_BYTES_BIG_ENDIAN 0
-
-#ifndef BFD_ASSEMBLER
- #error D10V support requires BFD_ASSEMBLER
-#endif
+#define TARGET_BYTES_BIG_ENDIAN 1
 
 /* The target BFD architecture.  */
 #define TARGET_ARCH bfd_arch_d10v
@@ -50,17 +45,10 @@ long md_pcrel_from_section (struct fix *, segT);
 #define md_number_to_chars	     number_to_chars_bigendian
 
 int d10v_cleanup (void);
-#define md_after_pass_hook()	     d10v_cleanup ()
+void d10v_frob_label (symbolS *);
 #define md_cleanup()		     d10v_cleanup ()
 #define md_do_align(a,b,c,d,e)	     d10v_cleanup ()
-#define tc_frob_label(sym)				\
-  do							\
-    {							\
-      d10v_cleanup ();					\
-      symbol_set_frag (sym, frag_now);			\
-      S_SET_VALUE (sym, (valueT) frag_now_fix ());	\
-    }							\
-  while (0)
+#define tc_frob_label(sym)	     d10v_frob_label (sym)
 
 #define tc_fix_adjustable(FIX) d10v_fix_adjustable(FIX)
 bfd_boolean d10v_fix_adjustable (struct fix *);
