@@ -1,11 +1,11 @@
 /* tc-ip2k.h -- Header file for tc-ip2k.c.
-   Copyright (C) 2000, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2000-2016 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -20,11 +20,6 @@
 
 #define TC_IP2K
 
-#ifndef BFD_ASSEMBLER
-/* Leading space so will compile with cc.  */
- #error IP2K support requires BFD_ASSEMBLER
-#endif
-
 #define LISTING_HEADER "IP2xxx GAS "
 
 /* The target BFD architecture.  */
@@ -64,7 +59,7 @@ extern int ip2k_force_relocation (struct fix *);
 #define tc_gen_reloc gas_cgen_tc_gen_reloc
 
 #define md_elf_section_flags ip2k_elf_section_flags
-extern int ip2k_elf_section_flags (int, int, int);
+extern int ip2k_elf_section_flags (flagword, bfd_vma, int);
 
 #define md_operand(x) gas_cgen_md_operand (x)
 extern void gas_cgen_md_operand (expressionS *);
