@@ -1,5 +1,5 @@
 /* tc-tic4x.c -- Assemble for the Texas Instruments TMS320C[34]x.
-   Copyright (C) 1997-2014 Free Software Foundation, Inc.
+   Copyright (C) 1997-2016 Free Software Foundation, Inc.
 
    Contributed by Michael P. Hayes (m.hayes@elec.canterbury.ac.nz)
 
@@ -17,12 +17,12 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 51 Franklin Street - Fifth Floor, 
+   the Free Software Foundation, 51 Franklin Street - Fifth Floor,
    Boston, MA 02110-1301, USA.  */
 /*
   TODOs:
   ------
-  
+
   o .align cannot handle fill-data-width larger than 0xFF/8-bits. It
     should be possible to define a 32-bits pattern.
 
@@ -124,7 +124,8 @@ typedef struct tic4x_insn
     unsigned int nchars;	/* This is always 4 for the C30.  */
     unsigned long opcode;	/* Opcode number.  */
     expressionS exp;		/* Expression required for relocation.  */
-    int reloc;			/* Relocation type required.  */
+    /* Relocation type required.  */
+    bfd_reloc_code_real_type reloc;
     int pcrel;			/* True if relocation PC relative.  */
     char *pname;		/* Name of instruction in parallel.  */
     unsigned int num_operands;	/* Number of operands in total.  */
@@ -195,10 +196,10 @@ const char comment_chars[] = ";";
 
 /* This array holds the chars that only start a comment at the beginning of
    a line.  If the line seems to have the form '# 123 filename'
-   .line and .file directives will appear in the pre-processed output. 
+   .line and .file directives will appear in the pre-processed output.
    Note that input_file.c hand checks for '#' at the beginning of the
    first line of the input file.  This is because the compiler outputs
-   #NO_APP at the beginning of its output. 
+   #NO_APP at the beginning of its output.
    Also note that comments like this one will always work.  */
 const char line_comment_chars[] = "#*";
 
@@ -250,7 +251,7 @@ tic4x_gen_to_words (FLONUM_TYPE flonum, LITTLENUM_TYPE *words, int precision)
      The code in this function is altered slightly to support floats
      with 31-bits mantissas, thus the documentation below may be a
      little bit inaccurate.
-     
+
      By Michael P. Hayes <m.hayes@elec.canterbury.ac.nz>
      Here is how a generic floating point number is stored using
      flonums (an extension of bignums) where p is a pointer to an
@@ -398,7 +399,7 @@ tic4x_gen_to_words (FLONUM_TYPE flonum, LITTLENUM_TYPE *words, int precision)
       /* +INF:  Replace with maximum float.  */
       if (precision == S_PRECISION)
 	words[0] = 0x77ff;
-      else 
+      else
 	{
 	  words[0] = 0x7f7f;
 	  words[1] = 0xffff;
@@ -415,7 +416,7 @@ tic4x_gen_to_words (FLONUM_TYPE flonum, LITTLENUM_TYPE *words, int precision)
       /* -INF:  Replace with maximum float.  */
       if (precision == S_PRECISION)
 	words[0] = 0x7800;
-      else 
+      else
         words[0] = 0x7f80;
       if (precision == E_PRECISION)
         words[2] = 0x8000;
@@ -617,8 +618,8 @@ tic4x_atof (char *str, char what_kind, LITTLENUM_TYPE *words)
   return return_value;
 }
 
-static void 
-tic4x_insert_reg (char *regname, int regnum)
+static void
+tic4x_insert_reg (const char *regname, int regnum)
 {
   char buf[32];
   int i;
@@ -633,8 +634,8 @@ tic4x_insert_reg (char *regname, int regnum)
 				   &zero_address_frag));
 }
 
-static void 
-tic4x_insert_sym (char *symname, int value)
+static void
+tic4x_insert_sym (const char *symname, int value)
 {
   symbolS *symbolP;
 
@@ -672,7 +673,7 @@ tic4x_expression_abs (char *str, offsetT *value)
   return s;
 }
 
-static void 
+static void
 tic4x_emit_char (char c, int b)
 {
   expressionS exp;
@@ -682,7 +683,7 @@ tic4x_emit_char (char c, int b)
   emit_expr (&exp, b);
 }
 
-static void 
+static void
 tic4x_seg_alloc (char *name ATTRIBUTE_UNUSED,
 		 segT seg ATTRIBUTE_UNUSED,
 		 int size,
@@ -707,13 +708,12 @@ tic4x_seg_alloc (char *name ATTRIBUTE_UNUSED,
 }
 
 /* .asg ["]character-string["], symbol */
-static void 
+static void
 tic4x_asg (int x ATTRIBUTE_UNUSED)
 {
   char c;
   char *name;
   char *str;
-  char *tmp;
 
   SKIP_WHITESPACE ();
   str = input_line_pointer;
@@ -727,24 +727,19 @@ tic4x_asg (int x ATTRIBUTE_UNUSED)
       return;
     }
   *input_line_pointer++ = '\0';
-  name = input_line_pointer;
-  c = get_symbol_end ();	/* Get terminator.  */
-  tmp = xmalloc (strlen (str) + 1);
-  strcpy (tmp, str);
-  str = tmp;
-  tmp = xmalloc (strlen (name) + 1);
-  strcpy (tmp, name);
-  name = tmp;
+  c = get_symbol_name (&name);	/* Get terminator.  */
+  str = xstrdup (str);
+  name = xstrdup (name);
   if (hash_find (tic4x_asg_hash, name))
     hash_replace (tic4x_asg_hash, name, (void *) str);
   else
     hash_insert (tic4x_asg_hash, name, (void *) str);
-  *input_line_pointer = c;
+  (void) restore_line_pointer (c);
   demand_empty_rest_of_line ();
 }
 
 /* .bss symbol, size  */
-static void 
+static void
 tic4x_bss (int x ATTRIBUTE_UNUSED)
 {
   char c;
@@ -759,8 +754,9 @@ tic4x_bss (int x ATTRIBUTE_UNUSED)
   current_subseg = now_subseg;	/* Save current subseg.  */
 
   SKIP_WHITESPACE ();
-  name = input_line_pointer;
-  c = get_symbol_end ();	/* Get terminator.  */
+  c = get_symbol_name (&name);	/* Get terminator.  */
+  if (c == '"')
+    c = * ++ input_line_pointer;
   if (c != ',')
     {
       as_bad (_(".bss size argument missing\n"));
@@ -807,11 +803,10 @@ tic4x_globl (int ignore ATTRIBUTE_UNUSED)
 
   do
     {
-      name = input_line_pointer;
-      c = get_symbol_end ();
+      c = get_symbol_name (&name);
       symbolP = symbol_find_or_make (name);
       *input_line_pointer = c;
-      SKIP_WHITESPACE ();
+      SKIP_WHITESPACE_AFTER_NAME ();
       S_SET_STORAGE_CLASS (symbolP, C_EXT);
       S_SET_EXTERNAL (symbolP);
       if (c == ',')
@@ -828,10 +823,10 @@ tic4x_globl (int ignore ATTRIBUTE_UNUSED)
 }
 
 /* Handle .byte, .word. .int, .long */
-static void 
+static void
 tic4x_cons (int bytes)
 {
-  register unsigned int c;
+  unsigned int c;
   do
     {
       SKIP_WHITESPACE ();
@@ -871,11 +866,11 @@ tic4x_cons (int bytes)
 }
 
 /* Handle .ascii, .asciz, .string */
-static void 
+static void
 tic4x_stringer (int append_zero)
 {
   int bytes;
-  register unsigned int c;
+  unsigned int c;
 
   bytes = 0;
   do
@@ -924,7 +919,7 @@ tic4x_stringer (int append_zero)
 }
 
 /* .eval expression, symbol */
-static void 
+static void
 tic4x_eval (int x ATTRIBUTE_UNUSED)
 {
   char c;
@@ -939,15 +934,14 @@ tic4x_eval (int x ATTRIBUTE_UNUSED)
       as_bad (_("Symbol missing\n"));
       return;
     }
-  name = input_line_pointer;
-  c = get_symbol_end ();	/* Get terminator.  */
+  c = get_symbol_name (&name);	/* Get terminator.  */
   tic4x_insert_sym (name, value);
-  *input_line_pointer++ = c;
+  (void) restore_line_pointer (c);
   demand_empty_rest_of_line ();
 }
 
 /* Reset local labels.  */
-static void 
+static void
 tic4x_newblock (int x ATTRIBUTE_UNUSED)
 {
   dollar_label_clear ();
@@ -955,7 +949,7 @@ tic4x_newblock (int x ATTRIBUTE_UNUSED)
 
 /* .sect "section-name" [, value] */
 /* .sect ["]section-name[:subsection-name]["] [, value] */
-static void 
+static void
 tic4x_sect (int x ATTRIBUTE_UNUSED)
 {
   char c;
@@ -967,11 +961,11 @@ tic4x_sect (int x ATTRIBUTE_UNUSED)
   SKIP_WHITESPACE ();
   if (*input_line_pointer == '"')
     input_line_pointer++;
-  section_name = input_line_pointer;
-  c = get_symbol_end ();	/* Get terminator.  */
+  c = get_symbol_name (&section_name);	/* Get terminator.  */
+  if (c == '"')
+    c = * ++ input_line_pointer;
   input_line_pointer++;		/* Skip null symbol terminator.  */
-  name = xmalloc (input_line_pointer - section_name + 1);
-  strcpy (name, section_name);
+  name = xstrdup (section_name);
 
   /* TI C from version 5.0 allows a section name to contain a
      subsection name as well. The subsection name is separated by a
@@ -980,13 +974,16 @@ tic4x_sect (int x ATTRIBUTE_UNUSED)
      Volker Kuhlmann  <v.kuhlmann@elec.canterbury.ac.nz>.  */
   if (c == ':')
     {
-      c = get_symbol_end ();	/* Get terminator.  */
+      char *subname;
+      c = get_symbol_name (&subname);	/* Get terminator.  */
+      if (c == '"')
+	c = * ++ input_line_pointer;
       input_line_pointer++;	/* Skip null symbol terminator.  */
       as_warn (_(".sect: subsection name ignored"));
     }
 
   /* We might still have a '"' to discard, but the character after a
-     symbol name will be overwritten with a \0 by get_symbol_end()
+     symbol name will be overwritten with a \0 by get_symbol_name()
      [VK].  */
 
   if (c == ',')
@@ -1014,7 +1011,7 @@ tic4x_sect (int x ATTRIBUTE_UNUSED)
 		 bfd_errmsg (bfd_get_error ()));
     }
 
-  /* If the last character overwritten by get_symbol_end() was an
+  /* If the last character overwritten by get_symbol_name() was an
      end-of-line, we must restore it or the end of the line will not be
      recognised and scanning extends into the next line, stopping with
      an error (blame Volker Kuhlmann <v.kuhlmann@elec.canterbury.ac.nz>
@@ -1026,7 +1023,7 @@ tic4x_sect (int x ATTRIBUTE_UNUSED)
 }
 
 /* symbol[:] .set value  or  .set symbol, value */
-static void 
+static void
 tic4x_set (int x ATTRIBUTE_UNUSED)
 {
   symbolS *symbolP;
@@ -1037,8 +1034,9 @@ tic4x_set (int x ATTRIBUTE_UNUSED)
       char c;
       char *name;
 
-      name = input_line_pointer;
-      c = get_symbol_end ();	/* Get terminator.  */
+      c = get_symbol_name (&name);	/* Get terminator.  */
+      if (c == '"')
+	c = * ++ input_line_pointer;
       if (c != ',')
 	{
 	  as_bad (_(".set syntax invalid\n"));
@@ -1056,7 +1054,7 @@ tic4x_set (int x ATTRIBUTE_UNUSED)
 }
 
 /* [symbol] .usect ["]section-name["], size-in-words [, alignment-flag] */
-static void 
+static void
 tic4x_usect (int x ATTRIBUTE_UNUSED)
 {
   char c;
@@ -1073,11 +1071,11 @@ tic4x_usect (int x ATTRIBUTE_UNUSED)
   SKIP_WHITESPACE ();
   if (*input_line_pointer == '"')
     input_line_pointer++;
-  section_name = input_line_pointer;
-  c = get_symbol_end ();	/* Get terminator.  */
+  c = get_symbol_name (&section_name);	/* Get terminator.  */
+  if (c == '"')
+    c = * ++ input_line_pointer;
   input_line_pointer++;		/* Skip null symbol terminator.  */
-  name = xmalloc (input_line_pointer - section_name + 1);
-  strcpy (name, section_name);
+  name = xstrdup (section_name);
 
   if (c == ',')
     input_line_pointer =
@@ -1122,7 +1120,7 @@ tic4x_usect (int x ATTRIBUTE_UNUSED)
 }
 
 /* .version cpu-version.  */
-static void 
+static void
 tic4x_version (int x ATTRIBUTE_UNUSED)
 {
   offsetT temp;
@@ -1139,7 +1137,7 @@ tic4x_version (int x ATTRIBUTE_UNUSED)
   demand_empty_rest_of_line ();
 }
 
-static void 
+static void
 tic4x_init_regtable (void)
 {
   unsigned int i;
@@ -1157,7 +1155,7 @@ tic4x_init_regtable (void)
     }
 }
 
-static void 
+static void
 tic4x_init_symbols (void)
 {
   /* The TI tools accept case insensitive versions of these symbols,
@@ -1188,7 +1186,7 @@ tic4x_init_symbols (void)
      Source: TI: TMS320C3x/C4x Assembly Language Tools User's Guide,
      1997, SPRU035C, p. 3-17/3-18.  */
   tic4x_insert_sym (".REGPARM", tic4x_reg_args);
-  tic4x_insert_sym (".MEMPARM", !tic4x_reg_args);	
+  tic4x_insert_sym (".MEMPARM", !tic4x_reg_args);
   tic4x_insert_sym (".BIGMODEL", tic4x_big_model);
   tic4x_insert_sym (".C30INTERRUPT", 0);
   tic4x_insert_sym (".TMS320xx", tic4x_cpu == 0 ? 40 : tic4x_cpu);
@@ -1214,7 +1212,7 @@ tic4x_init_symbols (void)
 }
 
 /* Insert a new instruction template into hash table.  */
-static int 
+static int
 tic4x_inst_insert (const tic4x_inst_t *inst)
 {
   static char prev_name[16];
@@ -1235,7 +1233,7 @@ tic4x_inst_insert (const tic4x_inst_t *inst)
 
 /* Make a new instruction template.  */
 static tic4x_inst_t *
-tic4x_inst_make (char *name, unsigned long opcode, char *args)
+tic4x_inst_make (const char *name, unsigned long opcode, const char *args)
 {
   static tic4x_inst_t *insts = NULL;
   static char *names = NULL;
@@ -1244,10 +1242,9 @@ tic4x_inst_make (char *name, unsigned long opcode, char *args)
   if (insts == NULL)
     {
       /* Allocate memory to store name strings.  */
-      names = (char *) xmalloc (sizeof (char) * 8192);
+      names = XNEWVEC (char, 8192);
       /* Allocate memory for additional insts.  */
-      insts = (tic4x_inst_t *)
-	xmalloc (sizeof (tic4x_inst_t) * 1024);
+      insts = XNEWVEC (tic4x_inst_t, 1024);
     }
   insts[iindex].name = names;
   insts[iindex].opcode = opcode;
@@ -1264,10 +1261,10 @@ tic4x_inst_make (char *name, unsigned long opcode, char *args)
 }
 
 /* Add instruction template, creating dynamic templates as required.  */
-static int 
+static int
 tic4x_inst_add (const tic4x_inst_t *insts)
 {
-  char *s = insts->name;
+  const char *s = insts->name;
   char *d;
   unsigned int i;
   int ok = 1;
@@ -1291,7 +1288,7 @@ tic4x_inst_add (const tic4x_inst_t *insts)
 	    {
 	      tic4x_inst_t *inst;
 	      int k = 0;
-	      char *c = tic4x_conds[i].name;
+	      const char *c = tic4x_conds[i].name;
 	      char *e = d;
 
 	      while (*c)
@@ -1335,7 +1332,7 @@ tic4x_inst_add (const tic4x_inst_t *insts)
 /* This function is called once, at assembler startup time.  It should
    set up all the tables, etc., that the MD part of the assembler will
    need.  */
-void 
+void
 md_begin (void)
 {
   int ok = 1;
@@ -1392,18 +1389,18 @@ md_begin (void)
   tic4x_init_symbols ();
 }
 
-void 
+void
 tic4x_end (void)
 {
-  bfd_set_arch_mach (stdoutput, bfd_arch_tic4x, 
+  bfd_set_arch_mach (stdoutput, bfd_arch_tic4x,
 		     IS_CPU_TIC4X (tic4x_cpu) ? bfd_mach_tic4x : bfd_mach_tic3x);
 }
 
-static int 
+static int
 tic4x_indirect_parse (tic4x_operand_t *operand,
 		      const tic4x_indirect_t *indirect)
 {
-  char *n = indirect->name;
+  const char *n = indirect->name;
   char *s = input_line_pointer;
   char *b;
   symbolS *symbolP;
@@ -1513,17 +1510,16 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
   input_line_pointer = s;
   SKIP_WHITESPACE ();
 
-  str = input_line_pointer;
-  c = get_symbol_end ();	/* Get terminator.  */
+  c = get_symbol_name (&str);	/* Get terminator.  */
   new_pointer = input_line_pointer;
   if (strlen (str) && (entry = hash_find (tic4x_asg_hash, str)) != NULL)
     {
-      *input_line_pointer = c;
+      (void) restore_line_pointer (c);
       input_line_pointer = (char *) entry;
     }
   else
     {
-      *input_line_pointer = c;
+      (void) restore_line_pointer (c);
       input_line_pointer = str;
     }
 
@@ -1662,7 +1658,7 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
   return new_pointer;
 }
 
-static int 
+static int
 tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *tinsn, int check)
 {
   const char *args = inst->args;
@@ -1824,7 +1820,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *tinsn, int check)
           if (!(operand->mode == M_REGISTER))
             break;
 	  reg = exp->X_add_number;
-	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+	  if ( (reg >= REG_R0 && reg <= REG_R7)
                || (IS_CPU_TIC4X (tic4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
 	    INSERTU (opcode, reg, 7, 0);
 	  else
@@ -1864,7 +1860,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *tinsn, int check)
 	  if (operand->mode != M_REGISTER)
 	    break;
 	  reg = exp->X_add_number;
-	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+	  if ( (reg >= REG_R0 && reg <= REG_R7)
                || (IS_CPU_TIC4X (tic4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
 	    INSERTU (opcode, reg, 15, 8);
 	  else
@@ -2055,7 +2051,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *tinsn, int check)
 	  if (operand->mode != M_REGISTER)
 	    break;
 	  reg = exp->X_add_number;
-	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+	  if ( (reg >= REG_R0 && reg <= REG_R7)
                || (IS_CPU_TIC4X (tic4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
 	    INSERTU (opcode, reg, 15, 0);
 	  else
@@ -2077,7 +2073,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *tinsn, int check)
 	  if (operand->mode != M_REGISTER)
 	    break;
 	  reg = exp->X_add_number;
-	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+	  if ( (reg >= REG_R0 && reg <= REG_R7)
                || (IS_CPU_TIC4X (tic4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
 	    INSERTU (opcode, reg, 20, 16);
 	  else
@@ -2336,7 +2332,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *tinsn, int check)
 static void
 tic4x_insn_check (tic4x_insn_t *tinsn)
 {
-  
+
   if (!strcmp (tinsn->name, "lda"))
     {
       if (tinsn->num_operands < 2 || tinsn->num_operands > 2)
@@ -2354,9 +2350,9 @@ tic4x_insn_check (tic4x_insn_t *tinsn)
            || !strcmp (tinsn->name, "ldf1_ldf2")
            || !strcmp (tinsn->name, "ldf2_ldf1") )
     {
-      if (tinsn->num_operands < 4 && tinsn->num_operands > 5 )
+      if (tinsn->num_operands < 4 || tinsn->num_operands > 5)
         as_fatal ("Illegal internal %s insn definition", tinsn->name);
-      
+
       if (tinsn->operands[1].mode == M_REGISTER
 	  && tinsn->operands[tinsn->num_operands-1].mode == M_REGISTER
 	  && tinsn->operands[1].expr.X_add_number == tinsn->operands[tinsn->num_operands-1].expr.X_add_number )
@@ -2364,7 +2360,7 @@ tic4x_insn_check (tic4x_insn_t *tinsn)
     }
 }
 
-static void 
+static void
 tic4x_insn_output (tic4x_insn_t *tinsn)
 {
   char *dst;
@@ -2389,7 +2385,7 @@ tic4x_insn_output (tic4x_insn_t *tinsn)
 }
 
 /* Parse the operands.  */
-static int 
+static int
 tic4x_operands_parse (char *s, tic4x_operand_t *operands, int num_operands)
 {
   if (!*s)
@@ -2410,13 +2406,14 @@ tic4x_operands_parse (char *s, tic4x_operand_t *operands, int num_operands)
 /* Assemble a single instruction.  Its label has already been handled
    by the generic front end.  We just parse mnemonic and operands, and
    produce the bytes of data and relocation.  */
-void 
+void
 md_assemble (char *str)
 {
   int ok = 0;
   char *s;
   int i;
   int parsed = 0;
+  size_t len;
   tic4x_inst_t *inst;		/* Instruction template.  */
   tic4x_inst_t *first_inst;
 
@@ -2426,7 +2423,7 @@ md_assemble (char *str)
       s = str;
       while (*s && *s != '|')
         s++;
-      
+
       if (*s && s[1]=='|')
         {
           if(insn->parallel)
@@ -2436,7 +2433,7 @@ md_assemble (char *str)
               insn->in_use = 0;
               return;
             }
-          
+
           /* Lets take care of the first part of the parallel insn */
           *s++ = 0;
           md_assemble(str);
@@ -2445,7 +2442,7 @@ md_assemble (char *str)
           /* .. and let the second run though here */
         }
     }
-  
+
   if (str && insn->parallel)
     {
       /* Find mnemonic (second part of parallel instruction).  */
@@ -2455,8 +2452,8 @@ md_assemble (char *str)
 	s++;
       if (*s)			/* Null terminate for hash_find.  */
 	*s++ = '\0';		/* and skip past null.  */
-      strcat (insn->name, "_");
-      strncat (insn->name, str, TIC4X_NAME_MAX - 1 - strlen (insn->name));
+      len = strlen (insn->name);
+      snprintf (insn->name + len, TIC4X_NAME_MAX - len, "_%s", str);
 
       insn->operands[insn->num_operands++].mode = M_PARALLEL;
 
@@ -2518,7 +2515,8 @@ md_assemble (char *str)
 	s++;
       if (*s)			/* Null terminate for hash_find.  */
 	*s++ = '\0';		/* and skip past null.  */
-      strncpy (insn->name, str, TIC4X_NAME_MAX - 3);
+      strncpy (insn->name, str, TIC4X_NAME_MAX - 1);
+      insn->name[TIC4X_NAME_MAX - 1] = '\0';
 
       if ((i = tic4x_operands_parse (s, insn->operands, 0)) < 0)
 	{
@@ -2547,7 +2545,7 @@ tic4x_cleanup (void)
    of chars emitted is stored in *sizeP.  An error message is
    returned, or NULL on OK.  */
 
-char *
+const char *
 md_atof (int type, char *litP, int *sizeP)
 {
   int prec;
@@ -2620,7 +2618,7 @@ md_atof (int type, char *litP, int *sizeP)
   return NULL;
 }
 
-void 
+void
 md_apply_fix (fixS *fixP, valueT *value, segT seg ATTRIBUTE_UNUSED)
 {
   char *buf = fixP->fx_where + fixP->fx_frag->fr_literal;
@@ -2664,7 +2662,7 @@ md_apply_fix (fixS *fixP, valueT *value, segT seg ATTRIBUTE_UNUSED)
 }
 
 /* Should never be called for tic4x.  */
-void 
+void
 md_convert_frag (bfd *headers ATTRIBUTE_UNUSED,
 		 segT sec ATTRIBUTE_UNUSED,
 		 fragS *fragP ATTRIBUTE_UNUSED)
@@ -2705,7 +2703,7 @@ md_estimate_size_before_relax (fragS *fragP ATTRIBUTE_UNUSED,
 
 
 int
-md_parse_option (int c, char *arg)
+md_parse_option (int c, const char *arg)
 {
   switch (c)
     {
@@ -2733,7 +2731,7 @@ md_parse_option (int c, char *arg)
       tic4x_reg_args = 0;
       break;
 
-    case 'r':			
+    case 'r':
       as_warn (_("Option -r is depreciated, please use -mregparm"));
     case OPTION_REGPARM:        /* register args */
       tic4x_reg_args = 1;
@@ -2792,7 +2790,7 @@ md_show_usage (FILE *stream)
 /* This is called when a line is unrecognized.  This is used to handle
    definitions of TI C3x tools style local labels $n where n is a single
    decimal digit.  */
-int 
+int
 tic4x_unrecognized_line (int c)
 {
   int lab;
@@ -2869,7 +2867,7 @@ md_section_align (segT segment ATTRIBUTE_UNUSED, valueT size)
   return size;			/* Byte (i.e., 32-bit) alignment is fine?  */
 }
 
-static int 
+static int
 tic4x_pc_offset (unsigned int op)
 {
   /* Determine the PC offset for a C[34]x instruction.
@@ -2946,7 +2944,7 @@ md_pcrel_from (fixS *fixP)
 
 /* Fill the alignment area with NOP's on .text, unless fill-data
    was specified. */
-int 
+int
 tic4x_do_align (int alignment,
 		const char *fill,
 		int len,
@@ -2954,7 +2952,7 @@ tic4x_do_align (int alignment,
 {
   /* Because we are talking lwords, not bytes, adjust alignment to do words */
   alignment += 2;
-  
+
   if (alignment != 0 && !need_pass_2)
     {
       if (fill == NULL)
@@ -2974,13 +2972,13 @@ tic4x_do_align (int alignment,
       else
 	frag_align_pattern (alignment, fill, len, max);
     }
-  
+
   /* Return 1 to skip the default alignment function */
   return 1;
 }
 
 /* Look for and remove parallel instruction operator ||.  */
-void 
+void
 tic4x_start_line (void)
 {
   char *s = input_line_pointer;
@@ -3013,9 +3011,9 @@ tc_gen_reloc (asection *seg ATTRIBUTE_UNUSED, fixS *fixP)
 {
   arelent *reloc;
 
-  reloc = (arelent *) xmalloc (sizeof (arelent));
+  reloc = XNEW (arelent);
 
-  reloc->sym_ptr_ptr = (asymbol **) xmalloc (sizeof (asymbol *));
+  reloc->sym_ptr_ptr = XNEW (asymbol *);
   *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixP->fx_addsy);
   reloc->address = fixP->fx_frag->fr_address + fixP->fx_where;
   reloc->address /= OCTETS_PER_BYTE;
