@@ -1,12 +1,11 @@
 /* flonum.h - Floating point package
-   Copyright 1987, 1990, 1991, 1992, 1994, 1996, 2000
-   Free Software Foundation, Inc.
+   Copyright (C) 1987-2016 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -16,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 /***********************************************************************\
  *									*
@@ -84,14 +83,14 @@ extern const int table_size_of_flonum_powers_of_ten;
  *									*
  \***********************************************************************/
 
-int atof_generic PARAMS ((char **address_of_string_pointer,
-			  const char *string_of_decimal_marks,
-			  const char *string_of_decimal_exponent_marks,
-			  FLONUM_TYPE * address_of_generic_floating_point_number));
+int atof_generic (char **address_of_string_pointer,
+		  const char *string_of_decimal_marks,
+		  const char *string_of_decimal_exponent_marks,
+		  FLONUM_TYPE * address_of_generic_floating_point_number);
 
-void flonum_copy PARAMS ((FLONUM_TYPE * in, FLONUM_TYPE * out));
-void flonum_multip PARAMS ((const FLONUM_TYPE * a, const FLONUM_TYPE * b,
-			    FLONUM_TYPE * product));
+void flonum_copy (FLONUM_TYPE * in, FLONUM_TYPE * out);
+void flonum_multip (const FLONUM_TYPE * a, const FLONUM_TYPE * b,
+		    FLONUM_TYPE * product);
 
 /***********************************************************************\
  *									*
