@@ -6,8 +6,8 @@ LITTLE_OUTPUT_FORMAT="elf32-littlenios2"
 BIG_OUTPUT_FORMAT="elf32-bignios2"
 TEXT_START_ADDR=0x2000
 OTHER_GOT_SYMBOLS='
-  _gp = ALIGN(16) + 0x7ff0;
-  PROVIDE(gp = _gp);
+  HIDDEN (_gp = ALIGN(16) + 0x7ff0);
+  PROVIDE_HIDDEN (gp = _gp);
 '
 ARCH=nios2
 MACHINE=
