@@ -1,6 +1,5 @@
 /* ldcref.c -- output a cross reference table
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-   2007, 2008  Free Software Foundation, Inc.
+   Copyright (C) 1996-2016 Free Software Foundation, Inc.
    Written by Ian Lance Taylor <ian@cygnus.com>
 
    This file is part of the GNU Binutils.
@@ -41,7 +40,8 @@
 /* We keep an instance of this structure for each reference to a
    symbol from a given object.  */
 
-struct cref_ref {
+struct cref_ref
+{
   /* The next reference.  */
   struct cref_ref *next;
   /* The object.  */
@@ -56,7 +56,8 @@ struct cref_ref {
 
 /* We keep a hash table of symbols.  Each entry looks like this.  */
 
-struct cref_hash_entry {
+struct cref_hash_entry
+{
   struct bfd_hash_entry root;
   /* The demangled name.  */
   const char *demangled;
@@ -66,7 +67,8 @@ struct cref_hash_entry {
 
 /* This is what the hash table looks like.  */
 
-struct cref_hash_table {
+struct cref_hash_table
+{
   struct bfd_hash_table root;
 };
 
@@ -161,7 +163,7 @@ add_cref (const char *name,
   struct cref_hash_entry *h;
   struct cref_ref *r;
 
-  if (! cref_initialized)
+  if (!cref_initialized)
     {
       if (!bfd_hash_table_init (&cref_table.root, cref_hash_newfunc,
 				sizeof (struct cref_hash_entry)))
@@ -198,24 +200,6 @@ add_cref (const char *name,
     r->def = TRUE;
 }
 
-/* Free the cref hash table.  */
-
-void
-cref_hash_table_free (void)
-{
-  if (cref_initialized)
-    {
-      bfd_hash_table_free (&cref_table.root);
-      cref_initialized = FALSE;
-      cref_symcount = 0;
-      old_table = NULL;
-      old_size = 0;
-      old_count = 0;
-      old_symcount = 0;
-      alloc_mark = NULL;
-    }
-}
-
 /* Called before loading an as-needed library to take a snapshot of
    the cref hash table, and after we have loaded or found that the
    library was not needed.  */
@@ -362,12 +346,15 @@ cref_fill_array (struct cref_hash_entry *h, void *data)
 static int
 cref_sort_array (const void *a1, const void *a2)
 {
-  const struct cref_hash_entry * const *p1 =
-      (const struct cref_hash_entry * const *) a1;
-  const struct cref_hash_entry * const *p2 =
-      (const struct cref_hash_entry * const *) a2;
+  const struct cref_hash_entry *const *p1
+    = (const struct cref_hash_entry *const *) a1;
+  const struct cref_hash_entry *const *p2
+    = (const struct cref_hash_entry *const *) a2;
 
-  return strcmp ((*p1)->demangled, (*p2)->demangled);
+  if (demangling)
+    return strcmp ((*p1)->demangled, (*p2)->demangled);
+  else
+    return strcmp ((*p1)->root.string, (*p2)->root.string);
 }
 
 /* Write out the cref table.  */
@@ -392,7 +379,7 @@ output_cref (FILE *fp)
     }
   fprintf (fp, _("File\n"));
 
-  if (! cref_initialized)
+  if (!cref_initialized)
     {
       fprintf (fp, _("No symbols\n"));
       return;
@@ -445,8 +432,16 @@ output_one_cref (FILE *fp, struct cref_hash_entry *h)
 	}
     }
 
-  fprintf (fp, "%s ", h->demangled);
-  len = strlen (h->demangled) + 1;
+  if (demangling)
+    {
+      fprintf (fp, "%s ", h->demangled);
+      len = strlen (h->demangled) + 1;
+    }
+  else
+    {
+      fprintf (fp, "%s ", h->root.string);
+      len = strlen (h->root.string) + 1;
+    }
 
   for (r = h->refs; r != NULL; r = r->next)
     {
@@ -464,7 +459,21 @@ output_one_cref (FILE *fp, struct cref_hash_entry *h)
 
   for (r = h->refs; r != NULL; r = r->next)
     {
-      if (! r->def)
+      if (r->common)
+	{
+	  while (len < FILECOL)
+	    {
+	      putc (' ', fp);
+	      ++len;
+	    }
+	  lfinfo (fp, "%B\n", r->abfd);
+	  len = 0;
+	}
+    }
+
+  for (r = h->refs; r != NULL; r = r->next)
+    {
+      if (!r->def && !r->common)
 	{
 	  while (len < FILECOL)
 	    {
@@ -484,7 +493,7 @@ output_one_cref (FILE *fp, struct cref_hash_entry *h)
 void
 check_nocrossrefs (void)
 {
-  if (! cref_initialized)
+  if (!cref_initialized)
     return;
 
   cref_hash_traverse (&cref_table, check_nocrossref, NULL);
@@ -525,8 +534,14 @@ check_local_sym_xref (lang_input_statement_type *statement)
 	    symname = sym->name;
 	  for (ncrs = nocrossref_list; ncrs != NULL; ncrs = ncrs->next)
 	    for (ncr = ncrs->list; ncr != NULL; ncr = ncr->next)
-	      if (strcmp (ncr->name, outsecname) == 0)
-		check_refs (symname, FALSE, sym->section, abfd, ncrs);
+	      {
+		if (strcmp (ncr->name, outsecname) == 0)
+		  check_refs (symname, FALSE, sym->section, abfd, ncrs);
+		/* The NOCROSSREFS_TO command only checks symbols defined in
+		   the first section in the list.  */
+		if (ncrs->onlyfirst)
+		  break;
+	      }
 	}
     }
 }
@@ -563,10 +578,16 @@ check_nocrossref (struct cref_hash_entry *h, void *ignore ATTRIBUTE_UNUSED)
 
   for (ncrs = nocrossref_list; ncrs != NULL; ncrs = ncrs->next)
     for (ncr = ncrs->list; ncr != NULL; ncr = ncr->next)
-      if (strcmp (ncr->name, defsecname) == 0)
-	for (ref = h->refs; ref != NULL; ref = ref->next)
-	  check_refs (hl->root.string, TRUE, hl->u.def.section,
-		      ref->abfd, ncrs);
+      {
+	if (strcmp (ncr->name, defsecname) == 0)
+	  for (ref = h->refs; ref != NULL; ref = ref->next)
+	    check_refs (hl->root.string, TRUE, hl->u.def.section,
+			ref->abfd, ncrs);
+	/* The NOCROSSREFS_TO command only checks symbols defined in the first
+	   section in the list.  */
+	if (ncrs->onlyfirst)
+	  break;
+      }
 
   return TRUE;
 }
@@ -574,7 +595,8 @@ check_nocrossref (struct cref_hash_entry *h, void *ignore ATTRIBUTE_UNUSED)
 /* The struct is used to pass information from check_refs to
    check_reloc_refs through bfd_map_over_sections.  */
 
-struct check_refs_info {
+struct check_refs_info
+{
   const char *sym_name;
   asection *defsec;
   struct lang_nocrossrefs *ncrs;
